--================================ TRIGGER PAIS =========================================
-- Control de la seqncia per assignar un IDE adequat a un nou PAIS
--=======================================================================================
CREATE OR REPLACE TRIGGER TRG_PAIS
BEFORE INSERT ON PAIS
FOR EACH ROW 
BEGIN
    IF :NEW.IDE_pais IS NULL THEN
      SELECT SEQ_PAIS.NEXTVAL INTO :NEW.IDE_pais FROM SYS.DUAL;
    END IF;
END;
/
--===============================TRIGGER CLIENT =========================================
-- Control de la seqncia per assignar un IDE adequat a un nou CLIENT
--========================================================================================
CREATE OR REPLACE TRIGGER TRG_CLIENTS
BEFORE INSERT ON CLIENT
FOR EACH ROW 
  BEGIN
    IF :NEW.IDE_client IS NULL THEN
      SELECT SEQ_CLIENT.NEXTVAL INTO :NEW.IDE_client FROM SYS.DUAL;
    END IF;
END;
/
--================================ TRIGGER CLIENT ESBORRAR ==============================
--Evitar que s'esborri de manera definitiva un client de la BD. 
--========================================================================================
create or replace trigger TRG_client_esborrar
before  
delete
on CLIENT
for each row
declare
  e_avis_esborrat_logic_no_fisic exception;
begin
  -- si s'intenta esborrar un CLIENT, el qu es fa s avisar que el qu cal s modificar el camp ACTIVASN de 'S' a 'N'  
  raise e_avis_esborrat_logic_no_fisic;
exception
  when e_avis_esborrat_logic_no_fisic then
    dbms_output.put_line('No es pot esborrar de manera definitiva el CLIENT, es recomana desactivar-lo');
    raise program_error;
end;
/
--======================= TRIGGER ORDRE ESBORRAR O MODIFICAR =============================
/*
El segent disparador s'encarrega de controlar les segents situacions:
- No es pot esborrar una ordre de manera definitiva/fsica, si de manera lgica posant el camp ACTIVASN al valor 'N'
- Una ordre amb estat executada o b cancellada, no podr ser modificada, excepte el camp ACTIVASN
*/
--========================================================================================
create or replace trigger TRG_ORDRE_ESBORRAR
BEFORE UPDATE or delete ON ORDRE
FOR EACH ROW 
declare
  EXC_AVIS1 EXCEPTION;
  EXC_AVIS2 EXCEPTION;
BEGIN
IF UPDATING THEN
    IF :OLD.ESTAT IN ('executada','cancelada') then
        if :new.activasn!='N' then
          RAISE EXC_AVIS1;
        END IF;
    END IF;
ELSIF DELETING THEN
    RAISE EXC_AVIS2;
END IF;
EXCEPTION
    WHEN EXC_AVIS1 THEN
       dbms_output.put_line('Una ordre "executada" o "cancellada" no es pot modificar');
       RAISE PROGRAM_ERROR;
    WHEN EXC_AVIS2 THEN
       dbms_output.put_line('No es pot esborrar una ORDRE de compre/venda');
       RAISE PROGRAM_ERROR;
end;
/ 
--======================= TRIGGER SUBSCRIPCIO_DATES_SENYAL ===============================
/*
-- Quan s'intenti donar d'alta una subscripci a un senyal de trading cal que la data sigui
-- igual o posterior a la data d'alta del senyal, per anterior a la data de fi de vigncia
*/
--========================================================================================
create or replace trigger TRG_SUBSCRIPCIO_DATES_SENYAL
BEFORE INSERT OR UPDATE OF DATA ON SUBSCRIPCIO
FOR EACH ROW 
declare
    V_DATAALTA DATE;
    V_FIVIGENCIA DATE;
    V_CLIENT CHAR(1);
    EXC_AVIS EXCEPTION;
    EXC_AVIS2 EXCEPTION;
BEGIN
    SELECT DATAALTA,FIVIGENCIA INTO v_dataalta,v_fivigencia
    FROM SENYAL_AUTOMATIC
    WHERE IDE_SENYAL=:NEW.SENYAL;
    IF  :NEW.DATA<V_DATAALTA or :NEW.DATA>V_FIVIGEnCIA then
        RAISE EXC_AVIS;
    END IF;
    SELECT ACTIVATSN INTO V_CLIENT
    FROM CLIENT
    WHERE IDE_CLIENT=:NEW.CLIENT;
    IF V_CLIENT='N' THEN
      RAISE EXC_AVIS2;
    END IF;
EXCEPTION
    WHEN EXC_AVIS THEN
       dbms_output.put_line('Data de subscripcio NO vlida, s posterior a la data d''alta del senyal o b al perode de vigncia');
       RAISE PROGRAM_ERROR;
    WHEN EXC_AVIS2 THEN
       dbms_output.put_line('No es pot subscriure a una senyal automtica un client que est donat de baixa!');
       RAISE PROGRAM_ERROR;
END;
/

--==================== ESTADSTICA 3 ###### CLIENTS AMB ACCI =====================================
/*
L'atribut "QPropietaris" emmagatzema el nmero de CLIENTS que tenen en la seva CARTERA cada ACCIO.

Per poder tenir aquest camp de la taula ACCIO sempre actualitzat s'han hagut de crear 1 
disparador.
*/           
-- Nmero de clients que tenen en la seva cartera lacci indicada com a parmetre	
--======================================================================================================
CREATE OR REPLACE TRIGGER TRG_CARTERA_QPROPIETARIS 
AFTER INSERT OR UPDATE ON CARTERA
FOR EACH ROW
DECLARE
BEGIN
  IF INSERTING THEN -- AL inserir un nou element a la cartera
        UPDATE ACCIO -- s'incrementa la quantitat de propietaris
        SET QPROPIETARIS=QPROPIETARIS+1
        WHERE TICKER LIKE :NEW.PRODUCTE AND :NEW.UNITATS>0;
  ELSIF UPDATING THEN -- Al modificar un paquet d'accions a la cartera
        IF :NEW.ACTIVASN = 'N' OR :NEW.UNITATS=0 THEN -- si es dna de baixa o b la quantitat d'unitats s 0
          UPDATE ACCIO -- cal decrementar la quantitat de propietaris
          SET QPROPIETARIS=QPROPIETARIS-1
          WHERE TICKER LIKE :NEW.PRODUCTE;
        END IF;
  END IF;
END;
/
--==================== ESTADSTICA 2 ###### ACCIONS DISPONIBLES =====================================
/*
L'atribut "unitatsEnVenda" emmagatzema el nmero d'accions disponibles, que sn les que estan en ordres de venda
en estat "pendent"

Per poder tenir aquest camp de la taula ACCIO sempre actualitzat s'han hagut de crear 3 
disparadors que crident un mateix procediment que s l'encarregat de fer l'acci necessaria sobre l'atribut.

Els 3 triggers dissenyats es disparen en les 3 situacions que impliquen possibles canvis sobre l'atribut
UNITATS EN VENDA:
- alta d'una nova ordre, DE trading automtic
- alta d'una nova ordre, DE trading discrecional
- modificaci d'un ordre
*/
--================================ TRIGGER ACCIONS UNITATS EN VENDA ====================================
CREATE OR REPLACE TRIGGER TRG_ORDRE_VENDA_PENDENTS_o
AFTER UPDATE ON ORDRE
FOR EACH ROW
DECLARE
V_ACCIO ACCIO.TICKER%TYPE;
BEGIN
if (:new.estat like 'pendent' AND :new.tipus like 'VENDA' and :new.activasn='N') or -- si es dna de baixa l'ordre
   (:old.estat like 'pendent' and :new.estat like 'cancelada' AND :new.tipus like 'VENDA') or -- si l'ordre passa a estar cancelada
   (:old.estat like 'pendent' and :new.estat like 'executada' AND :new.tipus like 'VENDA') then -- si l'ordre o passa a estar executada
         V_ACCIO:=func_ordre_Accio(:NEW.CLIENT,:NEW.IDE_ORDRE,:NEW.TRADING);
         proc_unitats_en_venda('O',V_ACCIO,:NEW.UNITATS);
end if;                
END;
/ 
CREATE OR REPLACE TRIGGER TRG_ORDRE_VENDA_PENDENTS_d 
AFTER INSERT ON T_DISCRECIONAL
FOR EACH ROW
declare
v_tipus ordre.tipus%type;
v_estat ordre.estat%type;
v_unitats ordre.unitats%type;
BEGIN
select tipus,estat,unitats INTO v_tipus,v_estat,v_unitats 
from ordre
where client = :new.client and ide_ordre = :new.ordre;
if v_tipus like 'VENDA' AND v_estat like 'pendent' then
        proc_unitats_en_venda('D',:new.accio,v_unitats);
end if;
END; 
/
CREATE OR REPLACE TRIGGER TRG_ORDRE_VENDA_PENDENTS_A
AFTER INSERT ON T_AUTOMATIC
FOR EACH ROW 
declare
v_tipus ordre.tipus%type;
v_estat ordre.estat%type;
v_unitats ordre.unitats%type;
V_ACCIO ACCIO.TICKER%TYPE;
BEGIN
select tipus,estat,unitats INTO v_tipus,v_estat,v_unitats 
from ordre
where client = :new.client and ide_ordre = :new.ordre;
if v_tipus like 'VENDA' AND v_estat like 'pendent' then
    SELECT PRODUCTE INTO V_ACCIO
    FROM SENYAL_AUTOMATIC 
    WHERE ide_senyal = :NEW.senyal;
    proc_unitats_en_venda('A',v_accio,v_unitats);
end if;
END;
/