--======================================================================================================
--                    1. ESTADSTICA 1 ###### BENEFICI
--======================================================================================================
/* 
Benefici net en euros i en percentatge de la cartera d'accions, a partir dun client i un mes indicats com a parmetres.
	valorIniCartera: valor en euros de totes les accions que t en cartera amb el preu de cotitzaci en el dia 1 del mes i any que correspongui
	valorFiCartera: valor en euros de totes les accions que t en cartera amb el preu de cotitzaci en el ltim dia del mes i any que correspongui
	p100Benefici: s un camp calculat, que es pot obtenir de fer la segent manera:
 (valorFiCartera-valorIniCartera)*100/valorFiCartera
Si el valor resultant s negatiu, enlloc de benefici hi hagut prdua.

S'han controlat les segents situacions:
- client no existent a la BD
- client donat de baixa
- mes no vlid
- any no vlid
- mes i any posterior als actuals
*/ 
CREATE OR REPLACE PROCEDURE PROC_EST_1 (P_CLIENT   IN CLIENT.IDE_CLIENT%TYPE,
                                        P_MES     IN EST_MES_CLIENT.MES%TYPE, --VARCHAR(15)
                                        P_ANY     IN EST_MES_CLIENT.ANNY%TYPE, --NUMBER(4,0)
                                        RSP OUT VARCHAR2,
                                        ERR_DESC OUT VARCHAR2) AS
  V_EUROS EST_MES_CLIENT.VALORINICARTERA%TYPE;
  V_PERCE EST_MES_CLIENT.p100Benefici%TYPE;
  V_PENTRADA auditoria.pentrada%type;
  V_PSORTIDA auditoria.psortida%type;
  type a_varray is varray(12) of est_mes_client.mes%type;
  v_mesos a_varray:=a_varray('GENER','FEBRER','MAR','ABRIL','MAIG','JUNY','JULIOL','AGOST','SETEMBRE','OCTUBRE','NOVEMBRE','DESEMBRE');
  v_i number;
  v_s char(1);
  v_data date;
  exc_client_no_exists exception;
  exc_baixa exception;
  exc_mes exception;
  exc_data_futura exception;
  exc_any exception;
BEGIN
  RSP:='OK - PROC_EST_1';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_CLIENT||'-'||P_MES||'-'||P_ANY;
  -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
  PROC_ALTA_AUDITORIA ('ESTADSTICA 1',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
  select count(*) into v_i
  from client
  where ide_client = p_client;
  if v_i = 0 then
    raise exc_client_no_exists;
  end if;
  select activatsn into v_s
  from client
  where ide_client = p_client;
  if v_s = 'N' then
    raise exc_baixa;
  end if;
  v_i:=1;
  while v_i<=12 and v_mesos(v_i) not like p_mes  loop
      v_i:=v_i+1;
  end loop;
  if v_i=13 then
    raise exc_mes;
  end if;
  IF P_ANY < 2015 THEN
      RAISE EXC_ANY;
  END IF;
  v_data:= sysdate;
  if (v_i>to_char(sysdate,'mm') and p_any=to_char(sysdate,'yyyy')) OR p_any>to_char(sysdate,'yyyy') then
    raise exc_data_futura;
  end if;
  SELECT valorFiCartera-valorIniCartera,p100Benefici into v_euros,v_perce
  FROM EST_MES_CLIENT
  WHERE CLIENT=P_CLIENT AND MES LIKE P_MES AND AnNY=P_ANY;
  dbms_output.put_line('El client '||p_client||' en el mes '||p_mes||' de l''any '||p_any);
  IF V_EUROS>0 THEN
      dbms_output.put_line('t un Benefici net en euros de '||v_euros||' i en percentatge --> '||v_perce||'%');
  ELSif v_euros<0 then
      v_euros:=v_Euros*(-1);
      v_perce:=v_perce*(-1);
      dbms_output.put_line('t unes Prdues netes en euros de '||v_euros||' i en percentatge --> '||v_perce||'%');
  else -- cas que els valors sn 0.
      dbms_output.put_line('no t accions en cartera');
  END IF;
EXCEPTION
  WHEN exc_client_no_exists THEN
      RSP := 'ERROR 150290';
  		ERR_DESC := 'El client no existeix a la BD';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN exc_baixa THEN
      RSP := 'ERROR 150290';
  		ERR_DESC := 'El client '||p_client||' est donat de baixa';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN exc_mes THEN
      RSP := 'ERROR 150290';
  		ERR_DESC := 'El mes indicat no s vlid';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN exc_data_futura THEN
      RSP := 'ERROR 150290';
  		ERR_DESC := 'El mes i/o any indicats sn posteriors a la data actual';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN exc_any THEN
      RSP := 'ERROR 150290';
  		ERR_DESC := 'L''any indicat no s valid';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150291';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--======================================================================================================
--                    2. ESTADSTICA 2 ###### ACCIONS DISPONIBLES
--======================================================================================================
/*
La segona estadstica que cal calcular sn el nmero daccions disponibles a partir del ticker d'una acci.
Aquest valor dins la Base de Dades s'ha emmagatzemat dins l'atribut 'unitatsEnVenda' de la taula ACCI,
i es pot definir com la suma de totes les accions que estan en ordres de venda en estat 'Pendent'.
*/
CREATE OR REPLACE PROCEDURE PROC_EST_2 (P_accio   IN ACCIO.TICKER%TYPE,
                                        RSP OUT VARCHAR2,
                                        ERR_DESC OUT VARCHAR2) iS
  v_q accio.unitatsenvenda%type;
  V_PENTRADA auditoria.pentrada%type;
  V_PSORTIDA auditoria.psortida%type;
BEGIN
  RSP:='OK - PROC_EST_2';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_accio;
  -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
  PROC_ALTA_AUDITORIA ('ESTADSTICA 2',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
  SELECT UNITATSenVenda into v_q
  FROM ACCIO
  WHERE ticker LIKE P_accio;
  dbms_output.put_line('De l''accio '||p_accio||' hi ha '||v_Q||' unitats disponibles');
EXCEPTION
  WHEN NO_DATA_FOUND THEN
      RSP := 'ERROR 150292';
  		ERR_DESC := 'El TICKER de l''accio no s''han trobat';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150293';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
end;
/
--======================================================================================================
--                    3. ESTADSTICA 3 ###### CLIENTS AMB ACCI
--======================================================================================================
/*
L'estadstica 3 ens demana tenir sempre disponible el clcul del 
nmero de clients que tenen en la seva cartera lacci indicada com a parmetre,
per aix s disposa de l'atribut 'qpropietaris' dins la taula ACCI 
*/
CREATE OR REPLACE PROCEDURE PROC_EST_3 (P_accio   IN ACCIO.TICKER%TYPE,
                                        RSP OUT VARCHAR2,
                                        ERR_DESC OUT VARCHAR2) iS
  v_q accio.qpropietaris%type;
  V_PENTRADA auditoria.pentrada%type;
  V_PSORTIDA auditoria.psortida%type;
BEGIN
  RSP:='OK - PROC_EST_3';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_accio;
  -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
  PROC_ALTA_AUDITORIA ('ESTADSTICA 3',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
  SELECT QPROPIETARIS into v_q
  FROM ACCIO 
  WHERE ticker like p_accio;
  IF V_Q > 0 THEN
    dbms_output.put_line('De l''accio '||p_accio||' hi ha '||v_Q||' clients que la tenen a la seva cartera');
  ELSE
    dbms_output.put_line('De l''accio '||p_accio||' NO HI HA CAP client que la tingui en la seva cartera');
  END IF;
EXCEPTION
  WHEN NO_DATA_FOUND THEN
      RSP := 'ERROR 150294';
  		ERR_DESC := 'El TICKER de l''accio no s''han trobat';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150295';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
end;
/
--======================================================================================================
--                    4. ESTADSTICA 4 ###### CLIENT AMB MS BENEFICI
--======================================================================================================
-- Client amb la cartera daccions amb un valor actual ms gran, juntament amb el valor en euros de la seva cartera	
CREATE OR REPLACE PROCEDURE PROC_EST_4 (P_data   IN date,
                                        RSP OUT VARCHAR2,
                                        ERR_DESC OUT VARCHAR2) iS
    v_CLIENT EST_BEST_CLIENT.CLIENT%type;
    V_VALOR EST_BEST_CLIENT.VALORCARTERA%TYPE;
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
BEGIN
    RSP:='OK - PROC_EST_4';
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_data;
    PROC_ALTA_AUDITORIA ('ESTADSTICA 4',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    SELECT client,valorCartera INTO v_client,v_valor
    FROM EST_BEST_CLIENT
    WHERE data = P_DATA;
    dbms_output.put_line('A data '||p_data||' el client '||v_client||' t la cartera ms gran amb un valor en euros de '||v_valor);

EXCEPTION
  WHEN NO_DATA_FOUND THEN
      RSP := 'ERROR 150296';
  		ERR_DESC := 'No s''ha trobat cap client amb aquesta data';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150297';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);    
end;
/
--======================================================================================================
--                    5. ESTADSTICA 5 ###### ACCI MS NEGOCIADA
--======================================================================================================
-- Acci ms negociada (comprada i venuda) de forma histrica juntament amb el seu volum
-- *Controlar el cas dempats	
CREATE OR REPLACE PROCEDURE PROC_EST_5 (P_data   IN date,
                                        RSP OUT VARCHAR2,
                                        ERR_DESC OUT VARCHAR2) iS
  v_ACCIO EST_ACCIOMESNEGO.accio%type; 
  V_VOLUM EST_ACCIOMESNEGO.unitatsnegociades%TYPE;
  V_PENTRADA auditoria.pentrada%type;
  V_PSORTIDA auditoria.psortida%type;
BEGIN
  RSP:='OK - PROC_EST_5';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_data;
  PROC_ALTA_AUDITORIA ('ESTADSTICA 5',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
  SELECT ACCIO,unitatsNegociades INTO V_ACCIO,V_VOLUM
  FROM EST_ACCIOMESNEGO 
  WHERE data = P_DATA;
  dbms_output.put_line('A data '||p_data||' l''acci ms negociada s: '||v_accio||' amb un volum de '||v_volum||' unitats negociades');
EXCEPTION
  WHEN NO_DATA_FOUND THEN
      RSP := 'ERROR 150297';
  		ERR_DESC := 'No s''ha trobat cap client amb aquesta data';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150298';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);    
end;
/ 