set define off;
--====================================================================================
------------ ALTA D'UNA CRIDA A UN PROCEDIMENT DINS LA TAULA AUDITORIA----------------
--====================================================================================
CREATE OR REPLACE PROCEDURE PROC_ALTA_AUDITORIA (P_PROCEDIMENT     IN AUDITORIA.PROCEDIMENT%TYPE, 
                                            P_ENTRADES     IN AUDITORIA.PENTRADA%TYPE,
                                            P_SORTIDES IN AUDITORIA.PSORTIDA%TYPE,
                                            P_RESULTAT IN AUDITORIA.RESULTATEXECUCIO%TYPE ) IS
BEGIN
INSERT INTO AUDITORIA (IDE_AUDITORIA,data,procediment,pentrada,psortida,resultatExecucio)
VALUES (SEQ_LOG.NEXTVAL,SYSDATE,P_PROCEDIMENT,P_ENTRADES,P_SORTIDES ,P_RESULTAT);
COMMIT;
END;
/
--====================================================================================
------------------ MODIFICACI D'UN REGISTRE DE LA TAULA AUDITORIA -------------------
--====================================================================================
CREATE OR REPLACE PROCEDURE PROC_MODIF_AUDITORIA (P_IDE     IN AUDITORIA.IDE_AUDITORIA%TYPE, 
                                            P_SORTIDES IN AUDITORIA.PSORTIDA%TYPE,
                                            P_RESULTAT IN AUDITORIA.RESULTATEXECUCIO%TYPE ) IS
BEGIN
  UPDATE AUDITORIA
  SET resultatExecucio = P_RESULTAT,
          PSORTIDA=P_SORTIDES
  WHERE IDE_AUDITORIA = P_IDE;
  COMMIT;
END;
/
--====================================================================================
---------------------------- ACCI QUE PERTANY A UNA ORDRE ---------------------------
--====================================================================================
-- Funci que retorna el ticker que pertany a una ordre a partir del seu identificador (ordre+client) i el tipus de trading
create or replace function func_ordre_Accio (p_client in number ,p_ordre in number,p_trading in char) return varchar2 is
v_trading char(1);
v_accio accio.TICKER%TYPE;
begin 
if (p_trading is null) then
  select trading into v_trading
  from ordre
  where ide_ordre = p_ordre and client = p_client;
else
  v_trading:=p_trading;
end if;
IF v_trading = 'A' then --si l'ordre s de TRADING AUTOMTIC
      SELECT SA.PRODUCTE INTO V_ACCIO
      FROM T_AUTOMATIC TA, SENYAL_AUTOMATIC SA 
      WHERE TA.CLIENT = P_CLIENT AND TA.ORDRE = P_ORDRE AND SA.IDE_senyal = TA.SENYAL;
ELSE -- TRADING DISCRECIONAL
      SELECT ACCIO INTO V_ACCIO
      FROM T_DISCRECIONAL
      WHERE CLIENT = P_CLIENT AND ORDRE = P_ORDRE;
END IF;
return v_accio;
end func_ordre_Accio;
/
--====================================================================================
------------------- ACTUALITZAR EL VALOR DE LES UNITATS EN VENDA ---------------------
--====================================================================================
-- Procediment que permet modificar una acci, en concret les unitats que hi ha disponibles
-- aquest noms s'executar quan s'activen uns disparadors dissenyats amb aquest objectiu
create or replace procedure  proc_unitats_en_venda (p_t in char,p_accio in varchar2,p_unitats in number) is
begin
    if (p_t='D' or p_t='A') then
          update accio
          set unitatsenvenda=unitatsenvenda+p_unitats -- hi ha ms unitats disponibles
          WHERE TICKER LIKE p_ACCIO;
    elsif (p_t='O') then
          update accio
          set unitatsenvenda=unitatsenvenda-p_unitats -- hi ha menys unitats disponibles
          WHERE TICKER LIKE p_ACCIO;
    end if;
end;
/
--====================================================================================
---------------------- INICIAR SEQNCIA----------------------------------------------
--====================================================================================
-- Aquest procediment te per objectiu comprovar que la seqncia creada per poder gestionar
-- els identificadors de la taula AUDITORIA estigui ja iniciada. 
-- En cas de no estar iniciada aquest procs s'encarregar de fer-ho sense provocar cap excepci.
-- Aquesta instrucci s la que donava error si no es crea aquest procediment: select seq_log.currval from dual;
create or replace procedure proc_iniciar_sequencia as
	EXC_seq_log EXCEPTION;
	PRAGMA EXCEPTION_INIT(EXC_seq_log, -08002);
  v_Actual number;
begin
  select seq_log.currval into v_Actual from dual;
EXCEPTION
	WHEN EXC_seq_log THEN
      select seq_log.nextval into v_Actual from dual;
end;
/

--=============================== AJUDA PEL CLCUL DE L'ESTADSTICA 1 ###### BENEFICI ==============================
/*
El segent procediment s'encarrega d'inserir un nou registre en la taula EST_MES_CLIENT, la qual proporciona la informaci 
de l'estadstica 1 que correspon al clcul del Benefici net en euros i en percentatge de la cartera d'accions
a partir dun client, un mes i un any indicats com a parmetres.
Aquest procediment fa us de la funci "func_VALOR_1_CARTERA" especificada a continuaci 
*/
--==================================================================================================================
--------------- CALCULAR VALOR DE LA CARTERA D'ACCIONS D'UN CLIENT -------------------
--==================================================================================================================
-- Aquesta funci t com a parmetres:
-- - una DATE i 
-- - un IDE de CLIENT 
-- Retorna el valor de la cartera del client especificat en la data especificada
CREATE OR REPLACE function func_VALOR_1_CARTERA (P_DATA IN DATE,
                                                        P_CLIENT IN NUMBER,
                                                        RSP       OUT VARCHAR2,
                                                        ERR_DESC  OUT VARCHAR2) return number IS
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
    v_control client.activatsn%type;
    V_VALOR NUMBER:=0;
BEGIN
    RSP:='OK - Calcular valorActualCartera del CLIENT '||P_CLIENT;
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_DATA;
    -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
    PROC_ALTA_AUDITORIA ('CALCULAR_VALOR_CARTERA_1_CLIENT',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    SELECT nvl(SUM(CARTERA.UNITATS*ACTUALITZAR.PREUTANCAMENT),0) INTO V_VALOR
    FROM CARTERA JOIN ACTUALITZAR ON (CARTERA.PRODUCTE = ACTUALITZAR.PRODUCTE)
    WHERE CLIENT = P_CLIENT AND ACTUALITZAR.DATA = P_DATA AND CARTERA.ACTIVASN='Y';
    return v_valor;
EXCEPTION
  WHEN OTHERS THEN
  		RSP := 'ERROR 150275';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--==============================================================================================================
--------------------- CALCULAR El BENEFICI NET D'UN CLIENT EN UN MES DETERMINAT --------------------------------
--==============================================================================================================
-- Aquest procediment t com a parmetres:
-- - un IDE de CLIENT 
-- - un MES en format numric (d'1 a 12)
-- - un ANY en format de 4 dgits
-- El seu objectiu s inserir un nou registre en la taula EST_MES_CLIENT
create or replace procedure PROC_CARTERA_MES(P_CLIENT IN NUMBER,
                                             P_MES IN NUMBER,
                                             P_ANY NUMBER,
                                             RSP       OUT VARCHAR2,
                                             ERR_DESC  OUT VARCHAR2) IS
    type a_varray is varray(12) of est_mes_client.mes%type;
    v_mesos a_varray:=a_varray('GENER','FEBRER','MAR','ABRIL','MAIG','JUNY','JULIOL','AGOST','SETEMBRE','OCTUBRE','NOVEMBRE','DESEMBRE');
    V_DATAINI DATE;
    V_DATAFI DATE;
    v_valor number;
    v_control client.activatsn%type;
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
    EXC_MES EXCEPTION;
    EXC_ANY EXCEPTION;
    EXC_BAIXA EXCEPTION;
    exc_cartera_buida exception;
BEGIN
    RSP:='OK - taula EST_MES_CLIENT ';
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_client||'-'||p_mes||'-'||p_any;
    -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
    PROC_ALTA_AUDITORIA ('ACTUALITZAR_taula_EST_MES_CLIENT',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    SELECT ACTIVATSN INTO V_CONTROL
    FROM CLIENT
    WHERE IDE_CLIENT = P_CLIENT;
    IF V_CONTROL = 'N' THEN
       RAISE EXC_BAIXA;
    END IF;
    select count(*) into v_valor
    from cartera
    where client=p_client and activasn='Y';
    if v_valor = 0 then
      RAISE EXC_CARTERA_BUIDA;
    end if;
    IF P_MES NOT between 1 AND 12 THEN
        RAISE EXC_MES;
    END IF;
    IF P_ANY < 2015 THEN
        RAISE EXC_ANY;
    END IF;
    V_DATAINI:=TO_DATE('01/'||P_MES||'/'||P_ANY,'DD/MM/YYYY');
    IF P_MES IN (4,6,9,11) THEN
        V_DATAFI:=TO_DATE('30/'||P_MES||'/'||P_ANY,'DD/MM/YYYY');
    ELSIF P_MES IN (1,3,5,7,8,10,12) THEN
        V_DATAFI:=TO_DATE('31/'||P_MES||'/'||P_ANY,'DD/MM/YYYY');
    ELSE -- MES DE FEBRER
        IF (MOD(P_ANY,4)=0 AND MOD(P_ANY,100)!=0) OR MOD(P_ANY,400)=0 THEN -- ANY DE TRASPS DE 366 DIES
            V_DATAFI:=TO_DATE('29/'||P_MES||'/'||P_ANY,'DD/MM/YYYY');
        ELSE -- ANY NORMAL DE 365 DIES
            V_DATAFI:=TO_DATE('28/'||P_MES||'/'||P_ANY,'DD/MM/YYYY');
        END IF;
    END IF;
    -- control que pel client, mes i any indicat ja s'havia calculat el benefici en ocasions anteriors
        select count(*) into v_valor
        from est_mes_client
        where client = p_client and anny = p_any and mes = v_mesos(p_mes);
        if v_valor = 0 then
            INSERT INTO EST_MES_CLIENT (CLIENT,ANNY,MES,P100BENEFICI,VALORINICARTERA,VALORFICARTERA)
            VALUES (P_CLIENT,P_ANY,V_MESOS(P_MES),0,0,0);
            v_valor:=func_VALOR_1_CARTERA (v_dataini,P_CLIENT,RSP,ERR_DESC);
              update est_mes_client
              set valorINIcartera=DECODE(V_VALOR,NULL,0,V_VALOR)
              where client = p_client and anny = p_any and mes = V_MESOS(p_mes);
            v_valor:=func_VALOR_1_CARTERA (v_datafi,P_CLIENT,RSP,ERR_DESC);
              update est_mes_client
              set valorFIcartera=DECODE(V_VALOR,NULL,0,V_VALOR)
              where client = p_client and anny = p_any and mes = V_MESOS(p_mes);
            if v_valor is not null and v_valor!=0 then
              update est_mes_client
              set P100BENEFICI=(v_valor-VALORINICARTERA)*100/v_valor
              where client = p_client and anny = p_any and mes = V_MESOS(p_mes);
            end if;        
            commit;
            --DBMS_OUTPUT.PUT_LINE(RSP);
        else
            DBMS_OUTPUT.PUT_LINE(RSP||' el client, mes i any indicat ja estaven calculats');
        end if;
EXCEPTION
  WHEN EXC_BAIXA THEN
  		RSP:= 'ERROR 150270'; 
    	ERR_DESC:= 'El client '||p_client||' est donat de baixa';
		  -- Actualitzem la taula AUDITORIA amb el valor dels parmetres de sortida
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_CARTERA_BUIDA THEN      
  		RSP:= 'ERROR 150271'; 
    	ERR_DESC:= 'El client '||p_client||' no t accions en cartera';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
   -- 	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN no_data_found THEN      
  		RSP:= 'ERROR 150271'; 
    	ERR_DESC:= 'El client '||p_client||' no existeix a la base de dades';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
 WHEN EXC_MES THEN
  		RSP:= 'ERROR 150272'; 
    	ERR_DESC:= 'El mes indicat no s correcte';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);         
  WHEN EXC_ANY THEN
  		RSP:= 'ERROR 150273'; 
    	ERR_DESC:= 'L''any indicat no s vlid';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);               
  WHEN OTHERS THEN
  		RSP := 'ERROR 150274';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/ 
--==========================ESTADSTICA 4 ###### CLIENT AMB MS BENEFICI============================================================================
-- manteniment de la taula EST_BEST_CLIENT amb l'objectiu de tenir sempre a punt el clcul del
-- Client amb la cartera daccions amb un valor actual ms gran, juntament amb el valor en euros de la seva cartera	i la data
-- Per aconseguir aquests valors es fan us de 2 procediments:
-- proc_BEST_CLIENT
-- proc_ACTUALITZAR_CARTERA
-- El 1r s'encarrega de (donada una data) cercar el client amb la cartera amb un valor ms alt i donar d'alta a la taula EST_BEST_CLIENT
-- El 2n procediment s'encarrega de (donada una data) actualitzar l'atribut VALORACTUALCARTERA per tots els clients
-- El 1r procediment crida al 2n per tenir les dades en la data que la interessi dins la taula CLIENT.
--==================================================================================================================================================
-------------------------------------------------- CLIENT AMB LA MILLOR CARTERA --------------------------------------------------------------------
--==================================================================================================================================================
-- Aquesta funci t un parmetre de tipus DATE
-- s el dia pel qual es vol saber el CLIENT QUE T LA CARTERA AMB UN IMPORT MS ALT
CREATE OR REPLACE procedure proc_BEST_CLIENT (P_DATA IN DATE,
                                            RSP       OUT VARCHAR2,
                                            ERR_DESC  OUT VARCHAR2) IS
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
    V_VALOR NUMBER:=0;
    V_CLIENT CLIENT.IDE_CLIENT%TYPE;
    EXC_DATA EXCEPTION;
    exc_no_clients exception;
BEGIN
    RSP:='OK - MILLOR CLIENT ';
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_DATA;
    -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
    PROC_ALTA_AUDITORIA ('MILLOR CLIENT',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    IF P_DATA IS NULL THEN
      RAISE EXC_DATA;
    END IF;
    select count(*) into v_valor
    from client
    where activatsn='Y';
    if v_valor=0 then
      raise exc_no_Clients;
    end if;
    select count(*) into v_valor
    from est_best_client
    where data=p_data;
    if v_valor=0 then -- si encara no s'ha calculat l'estadstica 4 per la data indicada
        proc_ACTUALITZAR_CARTERA (P_DATA,RSP,ERR_DESC);
        select ide_client into v_client
        from client
        where activatsn='Y' and valoractualcartera= (select max(valoractualcartera)
                                                    from client where activatsn='Y');
        select max(valoractualcartera) INTO V_VALOR
        from client where activatsn='Y';
        if (v_valor!=0) then
            INSERT INTO EST_BEST_CLIENT VALUES (P_DATA,V_CLIENT,V_VALOR);
            COMMIT;
        end if;
    end if;
    DBMS_OUTPUT.PUT_LINE(RSP);
EXCEPTION
  WHEN EXC_DATA THEN
  		RSP:= 'ERROR 150276'; 
    	ERR_DESC:= 'Cal indicar una data';
		  -- Actualitzem la taula AUDITORIA amb el valor dels parmetres de sortida
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  when too_many_rows then
		  RSP:= 'ERROR 150277'; 
    	ERR_DESC:= 'Existeix ms d''un client amb una cartera amb el mxim benecifi';
		  PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  when exc_no_clients then
		  RSP:= 'ERROR 150277'; 
    	ERR_DESC:= 'No hi ha clients en actiu a la BD';
		  PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN OTHERS THEN
  		RSP := 'ERROR 150277';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--====================================================================================
--------------CALCULAR VALOR DE LA CARTERA D'ACCIONS DE tots els CLIENTS--------------
--====================================================================================
-- Aquesta funci t un parmetre de tipus DATE
-- s el dia pel qual es vol saber el valor actual de la cartera de tots els clients
CREATE OR REPLACE procedure proc_ACTUALITZAR_CARTERA (P_DATA IN DATE,
                                            RSP       OUT VARCHAR2,
                                            ERR_DESC  OUT VARCHAR2) IS
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
    V_VALOR NUMBER:=0;
    EXC_DATA EXCEPTION;
    CURSOR C_CLIENTS IS SELECT IDE_CLIENT,VALORACTUALCARTERA 
                        FROM CLIENT 
                        WHERE ACTIVATSN='Y' and dataalta>=p_data
                        FOR UPDATE OF VALORACTUALCARTERA;
BEGIN
  --  RSP:='OK - ACTUALITZAR valorActualCartera CLIENTS ';
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_DATA;
    -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
    PROC_ALTA_AUDITORIA ('ACTUALITZAR_VALOR_CARTERA_CLIENTS',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    IF P_DATA IS NULL THEN
      RAISE EXC_DATA;
    END IF;
    FOR R_C IN C_CLIENTS LOOP
          SELECT SUM(CARTERA.UNITATS*ACTUALITZAR.PREUTANCAMENT) INTO V_VALOR
          FROM CARTERA JOIN ACTUALITZAR ON (CARTERA.PRODUCTE = ACTUALITZAR.PRODUCTE)
          WHERE CLIENT = R_C.IDE_CLIENT AND ACTUALITZAR.DATA = P_DATA AND CARTERA.ACTIVASN='Y';
          UPDATE CLIENT
          SET VALORACTUALCARTERA=decode(V_VALOR,null,0,v_valor)
          WHERE CURRENT OF C_CLIENTS;
    END LOOP;
    COMMIT; 
EXCEPTION
  WHEN EXC_DATA THEN
  		RSP:= 'ERROR 150278'; 
    	ERR_DESC:= 'Cal indicar una data';
		  -- Actualitzem la taula AUDITORIA amb el valor dels parmetres de sortida
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN OTHERS THEN
  		RSP := 'ERROR 150279';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--========================================= ESTADSTICA 5 ###### ACCI MS NEGOCIADA =====================================
/*
El segent procediment s'encarrega del manteniment de la taula EST_ACCIOMESNEGO 
amb l'objectiu de tenir sempre a punt el clcul de l' Acci ms negociada (comprada i venuda) juntament amb el seu volum
*/
--========================================================================================================================
CREATE OR REPLACE PROCEDURE PROC_UNI_NEGOCIADES (p_data in date,
                                            RSP       OUT VARCHAR2,
                                            ERR_DESC  OUT VARCHAR2) IS
    V_PENTRADA auditoria.pentrada%type;
    V_PSORTIDA auditoria.psortida%type;
    v_unitatsD ordre.unitats%type;
    v_unitatsA ordre.unitats%type;
    cursor c_accions is select ticker FROM accio;
    v_accioMax accio.ticker%type:=null;
    v_volumMax ordre.unitats%type := 0;
    exc_data exception;
    EXC_NO_NEGOCIACIO exception;
BEGIN
    RSP:='OK - manteniment EST_ACCIOMESNEGO ';
    ERR_DESC:=NULL;
    V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
    V_PENTRADA:=P_DATA;
    -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
    PROC_ALTA_AUDITORIA ('manteniment EST_ACCIOMESNEGO',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
    IF P_DATA IS NULL THEN
      RAISE EXC_DATA;
    END IF;
    select count(*) into v_unitatsD
    from EST_ACCIOMESNEGO
    where data = p_data;
    if v_unitatsD=0 then -- si encara no s'havien calculat l'estadstica 5 per aquesta data
          for r_a in c_accions loop
            SELECT nvl(SUM(UNITATS),0) into v_unitatsD
            FROM ORDRE O join t_discrecional t on (o.ide_ordre=t.ordre and o.client=t.client)
            WHERE O.ESTAT LIKE 'executada' and t.accio like r_a.ticker and o.dataexecucio=p_data;
            
            SELECT nvl(SUM(UNITATS),0) into v_unitatsA
            FROM ORDRE O join t_automatic t on (o.ide_ordre=t.ordre and o.client=t.client)
                         join senyal_automatic s on (t.senyal=s.ide_senyal)
            WHERE O.ESTAT LIKE 'executada' and s.producte like r_a.ticker and o.dataexecucio=p_data;
            if (v_unitatsD+v_unitatsA)>v_volumMax then
                v_accioMax:=r_a.ticker;
                v_volumMax:=v_unitatsD+v_unitatsA;
            end if;
          end loop;
          if (v_accioMax is not null) then -- en cas contrari vol dir que aquell dia no s'han negociat accions
            insert into est_acciomesnego (accio,data,unitatsnegociades)
            values (v_accioMax,p_data,v_volumMax);
            commit;
          else
            raise EXC_NO_NEGOCIACIO;
          end if;  
      end if;
EXCEPTION
  WHEN EXC_DATA THEN
  		RSP:= 'ERROR 150280'; 
    	ERR_DESC:= 'Cal indicar una data';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_NO_NEGOCIACIO THEN
  		RSP:= 'ERROR 150281'; 
    	ERR_DESC:= 'El dia '||p_data||' el broker no ha executat cap ordre';
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    	--DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN OTHERS THEN
  		RSP := 'ERROR 150282';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
      PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
      DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--============================================================================================================
----------------------- INSERIR UN NOU VALOR D'UNA ACCI EN EL TANCAMENT DE SESSI ---------------------------
--============================================================================================================
CREATE OR REPLACE procedure proc_actualitzar_cotitzacio ( P_ACCIO IN ACCIO.TICKER%TYPE,
                                                          P_DATA IN DATE,
                                                          P_VALOR IN NUMBER,
                                                          RSP       OUT VARCHAR2,
                                                          ERR_DESC  OUT VARCHAR2)  IS
begin
INSERT INTO ACTUALITZAR VALUES (P_ACCIO,P_DATA,P_VALOR);
COMMIT;
END;
/
--============================================================================================================
----------------------- ALTES DELS PREUS DE TANCAMENT DIARI DE TOTES LES ACCIONS DE LA bd --------------------
---------------------------------------------- && ------------------------------------------------------------
----------------------- CLCUL DEL CLIENT AMB MS BENEFICI EN LA SEVA CARTERA --------------------------------
------------------------------------ (PER TENIR L'ESTADSTICA 4) ---------------------------------------------
---------------------------------------------- && ------------------------------------------------------------
------------------------------- CLCUL DE L'ACCI MS NEGOCIADA CADA DIA -------------------------------------
--------------------------------------- (PER TENIR L'ESTADSTICA 5) ------------------------------------------
--============================================================================================================
-- El procediment PROC_COTITZACIONS_AL_TANCAR t les segents utilitats:
-- 1. Generaci aleatria de preus de tancament en totes les sessions des de l'inici d's de la BD fins la data actual
-- 2. Clcul del client amb ms benefici en la seva cartera tamb des de l'inici d's de la BD fins la data actual
-- 3. Clcul de l'acci ms negociada de manera diaria si hi hagut ordres executades el dia que correspongui
CREATE OR REPLACE PROCEDURE PROC_COTITZACIONS_AL_TANCAR IS
CURSOR C_ACCIONS IS SELECT TICKER FROM ACCIO;
V_DATA DATE:=DATE '2015-01-01'; -- DATA EN QU ES COMENCEN LES INSERCIONS A LA BD
V_CANVI NUMBER(38,6);
V_VALOR NUMBER(38,6);
V_ARA NUMBER(38,6);
V_RSP VARCHAR2(100); -- variable necessria per fer la crida al procediment proc_best_client
V_ERR_DESC VARCHAR2(400); -- variable necessria per fer la crida al procediment proc_best_client
BEGIN
V_DATA:=V_DATA+1;
WHILE (V_DATA<SYSDATE) LOOP
  FOR R_ACCIO IN C_ACCIONS LOOP
        V_CANVI:=ROUND(dbms_random.value(0,1));
        V_VALOR:=dbms_random.value(0.001,0.8) ;
        SELECT PREUTANCAMENT INTO V_ARA
        FROM ACTUALITZAR
        WHERE data=V_DATA-1 AND producte=R_ACCIO.TICKER;
        IF V_CANVI=0 THEN
                PROC_ACTUALITZAR_COTITZACIO(R_ACCIO.TICKER,V_DATA,V_ARA+V_VALOR,v_rsp,v_err_desc);
        ELSE
                if v_ara-v_valor<0 then
                  PROC_ACTUALITZAR_COTITZACIO(R_ACCIO.TICKER,V_DATA,V_ARA+V_VALOR,v_rsp,v_err_desc);                    
                else
                  PROC_ACTUALITZAR_COTITZACIO(R_ACCIO.TICKER,V_DATA,V_ARA-V_VALOR,v_rsp,v_err_desc);                    
                end if;
        END IF;
  END LOOP;
  proc_BEST_CLIENT (V_DATA,v_RSP,v_ERR_DESC); -- cridem el procediment per poder calcular en aquella data quin s el client amb ms benefici
/*
  EST_BEST_CLIENT(#data,#client,valorCartera)
- Cada dia a lactualitzar les cotitzacions de totes les accions, un procs sencarregar de calcular el valorActual en euros de la Cartera de cada client i emmagatzemar-lo dins el camp valorActualCartera de la taula CLIENT 
- dins la taula EST_BEST_CLIENT noms quedar enregistrat el valor ms gran, juntament amb el client o clients que tinguin la cartera amb aquest import.
*/
  PROC_UNI_NEGOCIADES (v_data,v_RSP,v_ERR_DESC); -- cridem el procediment per saber cada dia quina s l'acci ms negociada, si s que n'hi ha
/*
  EST_ACCIOMESNEGO(#accio,#data,unitatsNegociades)
 - s el nmero daccions comprades ms el nmero daccions venudes
*/
  V_DATA:=V_DATA+1;
END LOOP;
END;
/ 
--============================================================================================================
------------------------------ ALTES DELS BENEFICIS MENSUALS DELS CLIENTS  -----------------------------------
----------------------------- (PER TENIR L'ESTADSTICA 1 SEMPRE A PUNT) --------------------------------------
--============================================================================================================
/*        
ESTADSTICA 1 BENEFICI x MES
manteniment de la taula EST_MES_CLIENT 
clcul del Benefici net en euros i en percentatge de la cartera d'accions
a partir dun client, un mes i un any indicats com a parmetres.
	valorIniCartera: valor en euros de totes les accions que t en cartera amb el preu de cotitzaci en el dia 1 del mes i any que correspongui
	valorFiCartera: valor en euros de totes les accions que t en cartera amb el preu de cotitzaci en el ltim dia del mes i any que correspongui
	p100Benefici: s un camp calculat, que es pot obtenir de fer la segent manera:
(valorFiCartera-valorIniCartera)*100/valorFiCartera
Si el valor resultant s negatiu, enlloc de benefici hi hagut prdua.
*/
CREATE OR REPLACE procedure proc_ESTADISTICA_1 IS
    CURSOR C_CLIENTS(DIA DATE) IS SELECT IDE_CLIENT
                        FROM CLIENT 
                        WHERE ACTIVATSN='Y' and dataalta<=DIA;
    V_RSP VARCHAR2(100); -- variable necessria per fer la crida al procediment proc_cartera_mes
    V_ERR_DESC VARCHAR2(400); -- variable necessria per fer la crida al procediment proc_cartera_mes
    v_data date:=DATE '2015-01-01'; -- DATA EN QU ES COMENCEN LES INSERCIONS A LA BD
    v_mes number(2,0); -- valor entre 1 i 12
    v_any number(4,0);
    v_mesFI number(2,0);
    v_anyFI number(4,0);
BEGIN
    v_any:=2015; -- any en el qu s'inici l's de la BD
    v_mesFI:=to_char(sysdate,'mm'); --quedar-se amb el mes de la data actual
    v_anyFI:=to_char(sysdate,'yyyy'); --quedar-se amb l'any de la data actual
    while (v_any<=v_anyFI) loop
        v_mes:=1;
        while (v_mes<=12 and v_any!=v_anyFI) or (v_any=v_anyFI and v_mes<=v_mesFI) loop
            FOR R_CLIENT IN C_CLIENTS(to_date('01/'||v_mes||'/'||v_any,'DD/MM/YYYY')) LOOP
                PROC_CARTERA_MES(r_CLIENT.ide_client,v_MES,v_ANY,v_RSP,v_ERR_DESC);
            END LOOP;
            v_mes:=v_mes+1;
        end loop;
      v_any:=v_any+1;
    end loop;
END;
/