--====================================================================================
---------------------- ALTA D'UNA ORDRE ----------------------------------------------
--====================================================================================
-- De tots els atributs que t una ordre, al donar-se d'alta n'hi ha alguns que no es demanaran
-- Aquests atributs sn: ide_ordre, dataexecucio,datacancelacio i activasn
-- aquests atributs s'omplen amb valors de manera automtica duran el procediment
-- Aquest procediment tamb s'encarrega de donar d'alta el tipus de TRADING que correspongui
-- s a dir, donar d'alta un registre a la taula T_AUTOMATIC o T_DISCRECIONAL segons el valor del parmetre P_TRADING.
-- Tamb es controlen les segents situacions:
--     - En donar dalta una nova ordre aquesta pot estar en estat pendent o executada, per no "cancelada"
--     - La data d'entrada de la nova ordre ha de ser posterior o igual a la data d'alta del client propietari d'aquesta ordre
--     - En cas de ser una ordre de TIPUS VENTA es controla que el CLIENT implicat disposi de suficients unitats en la seva cartera
--       i si en t queden actualitzades en la seva CARTERA
--     - En cas de ser una ordre de TIPUS COMPRA en estat EXECUTADA s'acutalitza la cartera del client automticament
--     - Quan l'ordre s de TRADING Automtic i passi a estar EXECUTADA es comprovar que la data d'execuci estigui inclosa 
--       dins el perode de vigncia de la senyal de trading automtic a la que pertany.
CREATE OR REPLACE PROCEDURE PROC_ALTA_ORDRE(P_CLIENT    IN ORDRE.CLIENT%TYPE,
                                            P_DATAENTRADA IN ordre.dataentrada%TYPE,
                                            P_TIPUS     IN ORDRE.TIPUS%TYPE,
                                            P_TRADING   IN ORDRE.TRADING%TYPE,
                                            P_ESTAT     IN ORDRE.ESTAT%TYPE,
                                            P_PREU      IN ORDRE.PREU%TYPE,
                                            P_UNITATS   IN ORDRE.UNITATS%TYPE,
                                            P_PMINIM    IN ORDRE.PREUMINIM%TYPE,
                                            P_PRODUCTE  IN T_DISCRECIONAL.ACCIO%TYPE,
                                            P_SENYAL    IN T_AUTOMATIC.SENYAL%TYPE,
                                            RSP       OUT VARCHAR2,
                                            ERR_DESC  OUT VARCHAR2) AS
  EXC_PARAM_OBLIGATORI EXCEPTION;
  EXC_CLIENT_BAIXA EXCEPTION;
  EXEC_NO_SUBSCRIT EXCEPTION;
  EXEC_ESTAT_CANCEL EXCEPTION;
  EXEC_DATES EXCEPTION;
  EXEC_UNITATS_INSUFICIENTS EXCEPTION;
  EXC_FORA_VIGENCIA EXCEPTION;
  V_RSP VARCHAR2(100);
  V_ERR_DESC VARCHAR2(400);
  V_SUBSCRIPCIO NUMBER;
	V_DATAEXE DATE;
  V_CLIENTACTIU CLIENT.ACTIVATSN%TYPE;
  V_DATAALTACLIENT DATE;
  V_IDE ORDRE.IDE_ordre%TYPE;--identificador de l'ltima ordre que ha fet el client
  V_PENTRADA VARCHAR2(1000);
  V_PSORTIDA VARCHAR2(100);
  V_UNITATS CARTERA.UNITATSDISPONIBLES%TYPE; --controlar si el client disposa de suficients unitats en cartera per poder donar d'alta una nova ordre de venta
  V_UNI CARTERA.UNITATS%TYPE;
  V_OK_V CHAR(1):='N'; --controlar si cal o no actualitzar les 'unitatsdisponibles' en cartera de l'acci que es vol vendre
  V_OK_C CHAR(1):='N'; --ordre de COMPRA i en estat EXECUTADA, actualitzar CARTERA
  V_CARTERA NUMBER;
  V_PRODUCTE ACCIO.TICKER%TYPE;
  R_SENYAL SENYAL_AUTOMATIC%ROWTYPE;-- PER desar accio que es vol vendre i periode de vigencia
BEGIN 
  RSP:='OK - ORDRE alta';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_CLIENT||','||P_DATAENTRADA||','||P_TIPUS||','||P_TRADING||','||P_ESTAT||','||P_PREU||','||P_UNITATS||','||P_PMINIM||','||P_PRODUCTE||','||P_SENYAL;
  -- ALTA a la taula AUDITORIA d'una nova crida a un procediment
  PROC_ALTA_AUDITORIA ('ALTA_ORDRE',V_PENTRADA,V_PSORTIDA,RSP||' '||NVL(ERR_DESC,' '));
	IF (P_CLIENT IS NULL OR P_TIPUS IS NULL OR P_ESTAT IS NULL OR P_UNITATS IS NULL OR P_DATAENTRADA IS NULL OR P_TRADING IS NULL OR (P_TRADING='D' AND P_PRODUCTE IS NULL) OR (P_TRADING='A' AND P_SENYAL IS NULL)) THEN
		RAISE EXC_PARAM_OBLIGATORI;
	END IF;
  IF P_ESTAT LIKE 'pendent' THEN
    V_DATAEXE:=null;
  ELSIF P_ESTAT LIKE 'cancelada' THEN
    RAISE EXEC_ESTAT_CANCEL;
  ELSE -- L'ORDRE EST EXECUTADA EN EL MOMENT QUE ES DNA D'ALTA AL SISTEMA
    V_DATAEXE:=P_DATAENTRADA;
  END IF;
  select dataalta,ACTIVATSN into v_dataAltaClient,V_CLIENTACTIU
  from client 
  where ide_client like P_CLIENT;
  IF V_CLIENTACTIU = 'N' THEN
    RAISE EXC_CLIENT_BAIXA;
  END IF;
  IF P_dataentrada<v_dataAltaClient then
    raise EXEC_dates;
  END IF;
  SELECT COUNT(*) INTO V_IDE -- per trobar el nou IDE_ORDRE de l'ordre que es vol donar d'ALTA
  FROM ORDRE
  WHERE CLIENT = P_CLIENT;
  IF P_TRADING = 'A' THEN  -- COMPROVAR QUE EL CLIENT EST SUBSCRIT AL SENYAL en cas de ser un ORDRE de TRADING 'A' automtic
      SELECT COUNT(*) INTO V_SUBSCRIPCIO
      FROM SUBSCRIPCIO join SENYAL_AUTOMATIC ON (SUBSCRIPCIO.SENYAL = SENYAL_AUTOMATIC.IDE_SENYAL)
      WHERE CLIENT = P_CLIENT AND SENYAL = P_SENYAL;
      IF v_subscripcio = 0 then
            raise EXEC_NO_SUBSCRIT;
      END IF;
      -- CONEIXER DE QUIN PRODUCTE ES TRACTA SI L'ordre s AUTOMTICA
      SELECT SENYAL_AUTOMATIC.* INTO r_Senyal
      FROM SUBSCRIPCIO join SENYAL_AUTOMATIC ON (SUBSCRIPCIO.SENYAL = SENYAL_AUTOMATIC.IDE_SENYAL)
      WHERE CLIENT = P_CLIENT AND SENYAL = P_SENYAL;
      -- S'ha desat dins la variable r_senyal.producte el codi de l'acci pertanyen a l'ordre que es vol donar d'alta
      V_PRODUCTE:=R_SENYAL.PRODUCTE;
      IF P_ESTAT LIKE 'executada' and (v_dataexe not between r_senyal.inivigencia and r_senyal.fivigencia) THEN
          RAISE EXC_FORA_VIGENCIA;
      END IF;
  ELSE
      V_PRODUCTE:=P_PRODUCTE;
  END IF;      
  IF P_TIPUS LIKE 'VENDA' THEN -- en cas de ser una VENDA, controlar que es disposa de suficients unitats en CARTERA
    SELECT UNITATS,UNITATSdisponibles INTO V_UNI,V_UNITATS
    FROM CARTERA
    WHERE CLIENT = P_CLIENT AND producte = V_PRODUCTE AND ACTIVASN='Y';
    -- Dins la variable V_UNITATS es t la quantitat d'accions disponibles per poder vendre
    IF V_UNITATS>=P_UNITATS THEN
        V_OK_V:='Y';
    ELSE
        RAISE EXEC_UNITATS_INSUFICIENTS;
    END IF;
  ELSE -- en cas de ser una COMPRA i en ESTAT 'executada' s'actualitzar la cartera
    IF P_ESTAT LIKE 'executada' THEN
          V_OK_C:='Y';
    END IF;
  END IF;
	-- Insereix una nova fila a la taula ORDRE i al TRADING que correspongui
  INSERT INTO ORDRE (IDE_ordre,CLIENT,DATAENTRADA,TIPUS,TRADING,estat,dataexecucio,DATACANCELACIO,PREU,UNITATS,PREUMINIM)
  VALUES (V_IDE+1,P_CLIENT,P_DATAENTRADA,P_TIPUS,P_TRADING,P_ESTAT,V_DATAEXE,NULL,P_PREU,P_UNITATS,P_PMINIM);
  IF (P_TRADING = 'D') THEN
      INSERT INTO T_DISCRECIONAL VALUES (V_IDE+1,P_CLIENT,V_PRODUCTE);
  ELSE
      INSERT INTO T_AUTOMATIC VALUES (V_IDE+1,P_CLIENT,P_SENYAL); 
  END IF;
  COMMIT;
  IF V_OK_V='Y' THEN -- s'actualitza la CARTERA del client en cas de ser una ORDRE de VENDA
    IF P_ESTAT LIKE 'pendent' THEN
        -- es modifiquen les unitats disponibles, es decrementen amb les de la ORDRE DE VENDA PENDENT
        PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,0,NULL,0,NULL,1,V_UNITATS-P_UNITATS,v_RSP,v_ERR_DESC);
    ELSIF P_ESTAT LIKE 'executada' THEN
        -- es modifiquen les unitats i les unitats disponibles, es decrementen amb les de la ORDRE DE VENDA EXECUTADA
        PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,0,NULL,1,V_UNI-P_UNITATS,1,V_UNITATS-P_UNITATS,v_RSP,v_ERR_DESC);
    END IF;
  END IF;
  IF V_OK_C='Y' THEN --ordre de COMPRA i en estat EXECUTADA, actualitzar CARTERA
    SELECT COUNT(*) INTO V_CARTERA -- COMPROVAR que ja t en cartera aquesta accio, en cas contrari es donar d'alta
    FROM CARTERA
    WHERE CLIENT = P_CLIENT AND PRODUCTE = V_PRODUCTE AND ACTIVASN='Y';
    IF V_CARTERA>0 THEN
        PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,1,V_DATAEXE,1,V_UNI+P_UNITATS,1,V_UNITATS+P_UNITATS,v_RSP,v_ERR_DESC);
    ELSE -- CLIENT que no t en CARTERA l'acci que s'acaba de COMPRAR
        PROC_ALTA_CARTERA(V_PRODUCTE,V_DATAEXE,P_CLIENT,P_UNITATS,v_RSP,v_ERR_DESC);
    END IF;
  END IF;
  DBMS_OUTPUT.PUT_LINE(RSP);
EXCEPTION 
		WHEN EXC_PARAM_OBLIGATORI THEN
  		RSP:= 'ERROR 150213'; 
    	ERR_DESC:= 'Manquen indicar valors de camps obligatoris';
		-- Actualitzem la taula AUDITORIA amb el valor dels parmetres de sortida
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN NO_DATA_FOUND THEN 
  		RSP:= 'ERROR 150214'; 
    	ERR_DESC:= 'El client '||P_CLIENT||' no existeix a la Base de Dades';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXEC_NO_SUBSCRIT THEN 
  		RSP:= 'ERROR 150215'; 
    	ERR_DESC:= 'El client '||P_CLIENT||' no est subscrit a la senyal de Trading '||P_SENYAL;
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXEC_DATES THEN 
  		RSP:= 'ERROR 150216'; 
    	ERR_DESC:= 'La data d''entrada de l''ordre no s correcte. El client propietari d''aquesta encara no estava donat d''alta al sistema.';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXEC_ESTAT_CANCEL THEN 
  		RSP:= 'ERROR 150217'; 
    	ERR_DESC:= 'No s possible donar d''alta una ordre amb estat "cancelada"';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  when EXEC_UNITATS_INSUFICIENTS then
  		RSP:= 'ERROR 150218'; 
    	ERR_DESC:= 'El client '||P_CLIENT||' no te suficients accions de '||r_senyal.PRODUCTE||' per vendre';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_FORA_VIGENCIA THEN
  		RSP:= 'ERROR 150219'; 
    	ERR_DESC:= 'La data d''EXECUCI de l''ordre no s correcte, est fora del perode de VIGNCIA del senyal de TRADING';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_CLIENT_BAIXA THEN
  		RSP:= 'ERROR 150220'; 
    	ERR_DESC:= 'No es pot inserir una nova ordre perqu el client '||p_client||' est donat de baixa';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
  		RSP := 'ERROR 150220';
    	ERR_DESC := TO_CHAR(SQLCODE) || ' ' ||SQLERRM;
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--====================================================================================
---------------------- MODIFICACI D'UNA ORDRE ---------------------------------------
--====================================================================================
-- Els camps que es podran modificar sn: ESTAT, PREU, PREUMINIM
-- Els parmetres que el seu nom s'inicia amb la partcula P_MOD_ noms poden pendre els valors: 0 o 1
-- 1 vol dir que el segent parmetre es vol modificar
-- 0 vol dir que el segent parmetre no es vol modificar, per aix habitualment ser NULL
set define off;
CREATE OR REPLACE PROCEDURE PROC_MODIF_ORDRE (P_IDE     IN ORDRE.IDE_ordre%TYPE, P_CLIENT   IN ORDRE.CLIENT%TYPE,
      P_MOD_ESTAT IN NUMBER,    P_ESTAT     IN ORDRE.ESTAT%TYPE, 
      P_MOD_PREU IN NUMBER,     P_PREU      IN ORDRE.PREU%TYPE,
      P_MOD_PREUMINIM IN NUMBER,P_PMINIM    IN ORDRE.PREUMINIM%TYPE,
                                            RSP       OUT VARCHAR2,
                                            ERR_DESC  OUT VARCHAR2) AS
	V_RSP VARCHAR2(100);
  V_ERR_DESC VARCHAR2(400);
  V_PENTRADA VARCHAR2(1000);
  V_PSORTIDA VARCHAR2(100);
  EXC_IDExCERCAR EXCEPTION;
	EXC_PARAM_OBLIGATORI EXCEPTION;
  EXC_ESTAT EXCEPTION;
  EXC_PREU EXCEPTION;
  EXC_BAIXA EXCEPTION;
  EXC_NO_MODIF EXCEPTION;
  R_ORDRE ORDRE%ROWTYPE;
  V_CARTERA CHAR(1):='N';
  V_PRODUCTE ACCIO.TICKER%TYPE;
  V_DATA DATE;
  V_UNI NUMBER;
  V_UNIDISPO NUMBER;
BEGIN
  RSP:='OK - ORDRE modificaci ';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_IDE||','||P_CLIENT||','||P_MOD_ESTAT||'-'||P_ESTAT||','||P_MOD_PREU||'-'||P_PREU||','||P_MOD_PREUMINIM||'-'||P_PMINIM;
  PROC_ALTA_AUDITORIA ('MODIFICAR_ORDRE',V_PENTRADA,V_PSORTIDA,V_PSORTIDA);
  IF (P_IDE IS NULL OR P_CLIENT IS NULL) THEN
		RAISE EXC_IDExCERCAR; 
	END IF;
  IF ((P_MOD_ESTAT = 1 AND P_ESTAT IS NULL) OR (P_MOD_PREU = 1 AND P_PREU IS NULL) OR (P_MOD_PREUMINIM = 1 AND P_PMINIM IS NULL)) THEN
    RAISE EXC_PARAM_OBLIGATORI;
  END IF;
  IF (P_MOD_ESTAT=0 AND P_MOD_PREU=0 AND P_MOD_PREUMINIM=0 ) THEN
      RAISE EXC_NO_MODIF;
  END IF;
  IF (P_MOD_PREU=1 AND P_MOD_ESTAT=0) THEN
    RAISE EXC_PREU;  -- No es pot modificar el preu d'una ordre, sense indicar el nou estat
  END IF;
  IF (P_MOD_ESTAT=1 AND P_ESTAT LIKE 'executada' AND P_MOD_PREU =0) THEN
    RAISE EXC_ESTAT; -- Per executar una ordre cal indicar el preu al qual s'ha comprat/venut el paquet d'accions
  END IF;
  SELECT * INTO R_ORDRE
  FROM ORDRE
	WHERE IDE_ordre=P_IDE AND CLIENT = P_CLIENT;
  IF (P_ESTAT LIKE 'executada' or (P_ESTAT LIKE 'cancelada' and r_ordre.tipus like 'VENDA')) then
    V_CARTERA:='Y'; -- cal modificar la cartera del client si l'ordre ara passar a EXECUTADA o b es CANCELA una ordre VENDA
  END IF;
  if (p_estat like 'executada' and r_ordre.trading='D') then
    v_data:=r_ordre.dataentrada+2; -- si passa a executada i s una ordre de trading DISCRECIONAL posarem com a data d'execuci la data d'entrada ms DOS dies
  END IF;
  IF (R_ORDRE.ACTIVASN='Y') THEN -- ordre que es pot modificar perqu est ACTIVA
    IF v_cartera='Y' then -- cal modificar la CARTERA DEL CLIENT 
    -- cerquem l'acci
      IF r_ordre.trading = 'A' then --si l'ordre s de TRADING AUTOMTIC
          SELECT SENYAL_AUTOMATIC.PRODUCTE,senyal_automatic.fivigencia INTO V_PRODUCTE,v_data
          FROM T_AUTOMATIC JOIN SUBSCRIPCIO ON (t_automatic.senyal= subscripcio.senyal AND t_automatic.client=subscripcio.client)
                           JOIN SENYAL_AUTOMATIC ON (SUBSCRIPCIO.SENYAL = SENYAL_AUTOMATIC.IDE_SENYAL)
          WHERE T_AUTOMATIC.CLIENT= P_CLIENT AND T_AUTOMATIC.ORDRE= P_IDE;        
      else -- si l'ordre s de TRADING discrecional
          V_producte:=func_ordre_Accio (p_client,p_IDE,'D'); -- Funci que retorna el ticker que pertany a una ordre a partir del seu identificador (ordre+client) i el tipus de trading
      end if;
      -- S'ha desat dins la variable V_PRODUCTE el codi de l'acci pertanyen a l'ordre que s'ha modificat
      if r_ordre.tipus like 'COMPRA' THEN
          SELECT COUNT(*) INTO V_CARTERA -- COMPROVAR que ja t en cartera aquesta accio, en cas contrari es donar d'alta
          FROM CARTERA
          WHERE CLIENT = P_CLIENT AND PRODUCTE = v_producte AND ACTIVASN='Y';
          IF V_CARTERA>0 THEN
            SELECT UNITATS,UNITATSDISPONIBLES INTO V_UNI,V_UNIDISPO
            FROM CARTERA
            WHERE CLIENT = P_CLIENT AND PRODUCTE = v_producte AND ACTIVASN='Y';
            PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,0,NULL,1,V_UNI+R_ORDRE.UNITATS,1,V_UNIDISPO+R_ORDRE.UNITATS,v_RSP,v_ERR_DESC);
          ELSE -- CLIENT que no t en CARTERA l'acci que s'acaba de COMPRAR
            PROC_ALTA_CARTERA(V_PRODUCTE,V_DATA,P_CLIENT,R_ORDRE.UNITATS,v_RSP,v_ERR_DESC);
          END IF;
      ELSE -- ordre de VENTA. Les unitats en la cartera ja estaven reservades per poder ser venudes
            SELECT UNITATS,UNITATSDISPONIBLES INTO V_UNI,V_UNIDISPO
            FROM CARTERA
            WHERE CLIENT = P_CLIENT AND PRODUCTE = v_producte AND ACTIVASN='Y';
        -- ara es procedeix a modificar la cartera del client afectat
        IF (P_ESTAT LIKE 'executada') then
            PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,0,NULL,1,V_UNI-R_ORDRE.UNITATS,0,NULL,v_RSP,v_ERR_DESC);
        end if;
        if p_estat like 'cancelada' then
            PROC_MODIF_CARTERA(P_CLIENT,V_PRODUCTE,0,NULL,0,NULL,1,V_UNIDISPO+R_ORDRE.UNITATS,v_RSP,v_ERR_DESC);
        end if;
      END IF;
    end if;
  -- ara es procedeix a modificar l'ordre
   IF (p_estat like 'executada') then
      UPDATE ORDRE
      SET ESTAT = DECODE(P_MOD_ESTAT,NULL,R_ORDRE.ESTAT,0,R_ORDRE.ESTAT,P_ESTAT),
        PREU = DECODE(P_MOD_PREU,NULL,R_ORDRE.PREU,0,R_ORDRE.PREU,P_PREU),
        PREUMINIM = DECODE(P_MOD_PREUMINIM, NULL, R_ORDRE.PREUMINIM,  0, R_ORDRE.PREUMINIM, P_PMINIM),
        dataexecucio = v_data
      WHERE IDE_ordre=P_IDE AND CLIENT = P_CLIENT;
   elsIF P_ESTAT LIKE 'cancelada' then
      UPDATE ORDRE
      SET ESTAT = DECODE(P_MOD_ESTAT,NULL,R_ORDRE.ESTAT,0,R_ORDRE.ESTAT,P_ESTAT),
        PREU = DECODE(P_MOD_PREU,NULL,R_ORDRE.PREU,0,R_ORDRE.PREU,P_PREU),
        PREUMINIM = DECODE(P_MOD_PREUMINIM, NULL, R_ORDRE.PREUMINIM,  0, R_ORDRE.PREUMINIM, P_PMINIM),
        datacancelacio = V_DATA
      WHERE IDE_ordre=P_IDE AND CLIENT = P_CLIENT;
   else
      UPDATE ORDRE
      SET ESTAT = DECODE(P_MOD_ESTAT,NULL,R_ORDRE.ESTAT,0,R_ORDRE.ESTAT,P_ESTAT),
        PREU = DECODE(P_MOD_PREU,NULL,R_ORDRE.PREU,0,R_ORDRE.PREU,P_PREU),
        PREUMINIM = DECODE(P_MOD_PREUMINIM, NULL, R_ORDRE.PREUMINIM,  0, R_ORDRE.PREUMINIM, P_PMINIM)
      WHERE IDE_ordre=P_IDE AND CLIENT = P_CLIENT;
   end if;
   COMMIT;
  ELSE -- l'ordre va ser donada de baixa, per tant, ara no es pot modificar
    RAISE EXC_BAIXA;
  END IF;
	DBMS_OUTPUT.PUT_LINE(RSP);
EXCEPTION
	WHEN EXC_IDExCERCAR THEN
		RSP:= 'ERROR 150221';
		ERR_DESC:= 'Identificador de l''ORDRE i/o CLIENT no indicats i sn obligatoris';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN EXC_PARAM_OBLIGATORI THEN
		RSP := 'ERROR 150222';
		ERR_DESC := 'Parmetres obligatoris no informats';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN NO_DATA_FOUND THEN  
		RSP := 'ERROR 150223';
		ERR_DESC := 'No s''ha trobat cap ORDRE amb IDE: '||P_IDE||' i CLIENT: '||P_CLIENT;
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_PREU THEN
		RSP := 'ERROR 150224';
		ERR_DESC := 'No es pot modificar el preu d''una ordre, a menys que passi a estar executada';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_ESTAT THEN
		RSP := 'ERROR 150225';  
		ERR_DESC := 'Per executar una ordre cal indicar el preu al qual s''ha comprat/venut el paquet d''accions';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_BAIXA THEN
  	RSP := 'ERROR 150226';
		ERR_DESC := 'No es pot modificar una ordre si ha estat donada de baixa';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
  WHEN EXC_NO_MODIF THEN
  	RSP := 'ERROR 150227';
		ERR_DESC := 'No s''ha indicat cap dada per ser modificada!';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN 
		RSP := 'ERROR 150227';
		ERR_DESC := 'ERROR '||SQLCODE||' '||SQLERRM;
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/
--SHOW ERRORS;
--====================================================================================
---------------------- BAIXA D'UNA ORDRE ---------------------------------------
--====================================================================================
-- La baixa no es far fsica, sin lgica, s a dir es posar l'atribut activaSN a 'N'
-- Aquest procediment tamb s'encarregar d'ACTUALITZAR el valor del camp UNITATSDISPONIBLES de la taula CARTERA
-- si l'ordre que s'ha donat de baixa s de tipus VENDA i el seu estat era PENDENT
CREATE OR REPLACE PROCEDURE PROC_BAIXA_ORDRE (P_IDE     IN ORDRE.IDE_ordre%TYPE,
                                              P_CLIENT  IN ORDRE.client%TYPE, 
                                              RSP       OUT VARCHAR2,
                                              ERR_DESC  OUT VARCHAR2) AS
	V_RSP VARCHAR2(100);
  V_ERR_DESC VARCHAR2(400);
  V_PSORTIDA VARCHAR2(100);
  V_PENTRADA VARCHAR2(1000);
  V_ACCIO accio.ticker%type;
  V_UNIDISPO CARTERA.UNITATSDISPONIBLES%TYPE;
	EXC_MANDATORY_IN EXCEPTION;
	EXC_ORDRE_JA_BAIXA EXCEPTION;
	R_ORDRE ORDRE%ROWTYPE;
BEGIN
  RSP:='OK - ORDRE baixa';
  ERR_DESC:=NULL;
  V_PSORTIDA:=RSP||' '||NVL(ERR_DESC,' ');
  V_PENTRADA:=P_IDE||' '||P_client;
  PROC_ALTA_AUDITORIA ('BAIXA_ORDRE',V_PENTRADA,V_PSORTIDA,V_PSORTIDA);
	IF (P_IDE IS NULL or P_client IS NULL) THEN
		RAISE EXC_MANDATORY_IN;
	END IF;
  SELECT * INTO R_ORDRE
  FROM ORDRE
  WHERE IDE_ordre = P_IDE and client = p_client;
	IF (R_ORDRE.ACTIVASN LIKE 'N') THEN
		RAISE EXC_ORDRE_JA_BAIXA;
	END IF;
  -- Modificaci de l'ORDRE posant ACTIVASN a N
  UPDATE ORDRE
	SET  ACTIVASN='N'
	WHERE IDE_ordre = P_IDE and client = p_client;
  -- si l'ordre que es dna de baixa era de venda i en estat pendent, caldr actualitzar la cartera, donat que tenia unes unitats reservades
  IF R_ORDRE.TIPUS LIKE 'VENDA' AND R_ORDRE.ESTAT LIKE 'pendent' THEN
   -- cercant l'acci afectada 
    IF (R_ORDRE.TRADING = 'D') THEN -- TRADING DISCRECIONAL
      V_ACCIO:=func_ordre_Accio (p_client,p_IDE,'D'); -- Funci que retorna el ticker que pertany a una ordre a partir del seu identificador (ordre+client) i el tipus de trading
    ELSE -- TRADING AUTOMTIC
      V_ACCIO:=func_ordre_Accio (p_client,p_IDE,'A');-- Funci que retorna el ticker que pertany a una ordre a partir del seu identificador (ordre+client) i el tipus de trading
    END IF;
    -- actualitzant la cartera, ara es tindran ms unitats disponibles, ja que no cal que estiguin reservades
    SELECT UNITATSDISPONIBLES INTO V_UNIDISPO
    FROM CARTERA
    WHERE CLIENT = P_CLIENT AND PRODUCTE = v_ACCIO AND ACTIVASN='Y';
    PROC_MODIF_CARTERA(P_CLIENT,V_ACCIO,0,NULL,0,NULL,1,V_UNIDISPO+R_ORDRE.UNITATS,v_RSP,v_ERR_DESC);
  END IF;
  COMMIT;
  DBMS_OUTPUT.PUT_LINE(RSP);
EXCEPTION
	WHEN EXC_MANDATORY_IN THEN
		RSP := 'ERROR 150228';
		ERR_DESC := 'L''IDE del ORDRE i/o el CLIENT sn obligatoris';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN NO_DATA_FOUND THEN
		RSP := 'ERROR 150229';
		ERR_DESC := 'L''IDE indicat o el NIF no corresponen a cap ORDRE';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN EXC_ORDRE_JA_BAIXA THEN
		RSP := 'ERROR 150230';
		ERR_DESC := 'L''ORDRE ja est donada de baixa';
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
	WHEN OTHERS THEN
		RSP := 'ERROR 150230';
		ERR_DESC:='ERROR '||SQLCODE||' '||SQLERRM;
    PROC_MODIF_AUDITORIA (SEQ_LOG.CURRVAL,'KO', RSP||' '||ERR_DESC);
    DBMS_OUTPUT.PUT_LINE(RSP||' '||ERR_DESC);
END;
/